"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.extendObjs = exports.wrapObj = exports.getPartialProperty = exports.flatObject = exports.deepClone = exports.assignDeep = exports.assignValids = exports.getObjectNest = void 0;
const types_1 = require("./types");
/** 根据 嵌套属性路径 获取嵌套对象的值 */
function getObjectNest(obj, path) {
    if (path.length > 0 && types_1.isObject(obj)) {
        let current = obj;
        for (let key of path) {
            current = current[key];
            if (current === undefined)
                return current;
        }
        return current;
    }
    return obj;
}
exports.getObjectNest = getObjectNest;
/** 合并有效数据，删除值为 undefined, null, NaN, 空字符串  */
function assignValids(...params) {
    let target = Object.assign({}, ...params);
    Object.keys(target).forEach(key => types_1.isEmpty(target[key]) && (delete target[key]));
    return target;
}
exports.assignValids = assignValids;
/** 根据路径设置 深层嵌套 的属性 */
function assignDeep(path, target, value, forceAssign) {
    if (path.length === 0)
        return;
    let setTarget = target;
    path = path.concat();
    while (path.length > 1) {
        let key = path.shift();
        if (types_1.isUndefined(setTarget[key])) {
            if (forceAssign) {
                setTarget[key] = {};
            }
            else {
                return;
            }
        }
        setTarget = setTarget[key];
    }
    setTarget[path[0]] = value;
}
exports.assignDeep = assignDeep;
/** 深度拷贝，仅支持属性 */
function deepClone(target) {
    return JSON.parse(JSON.stringify(target));
}
exports.deepClone = deepClone;
function _flatObject(obj, flatKey, clear) {
    Object.isExtensible(obj)
        && typeof obj[flatKey] === 'object'
        && Object.keys(obj[flatKey]).forEach(key => obj[key] = obj[flatKey][key]);
    clear && delete obj[flatKey];
}
/** 浅层扁平化对象 */
function flatObject(obj, keys, clear = true) {
    Array.isArray(keys)
        ? keys.forEach(key => _flatObject(obj, key, clear))
        : _flatObject(obj, keys, clear);
}
exports.flatObject = flatObject;
/** 指定属性值 组成新的对象返回，clone 表示返回的是全新的对象 */
function getPartialProperty(obj, keys, clone = false) {
    let result = {};
    keys.forEach(key => result[key] = obj[key]);
    return clone ? deepClone(result) : result;
}
exports.getPartialProperty = getPartialProperty;
/** 包裹 对象的属性 到指定到嵌套对象属性里 */
function wrapObj(obj, wrapField, excludes) {
    const wrap = {};
    Object.keys(obj).forEach(key => {
        wrap[key] = obj[key];
        excludes.includes(key) || Reflect.deleteProperty(obj, key);
    });
    obj[wrapField] = wrap;
}
exports.wrapObj = wrapObj;
function extendObjs(objs, source, wrapField) {
    let keys = Array.isArray(source[0]) ? source : [source];
    let excludes = keys.map(v => v[1]);
    objs.forEach(obj => {
        wrapField && wrapObj(obj, wrapField, excludes);
        keys.forEach(key => obj[key[1]] = wrapField
            ? obj[wrapField][key[0]]
            : obj[key[0]]);
    });
}
exports.extendObjs = extendObjs;
