"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.dateFormat = exports.secondsFormat = void 0;
const secondsConstant = {
    seconds: 1,
    minutes: 60,
    hour: 3600,
    day: 86400
};
const timePlaceholder = {
    year: 'Y+',
    Q: 'Q+',
    month: 'M+',
    day: 'D+',
    hour: 'H+',
    minute: 'm+',
    seconds: 's+',
    milliseconds: 'S'
};
function replaceTimePlaceholder(placeholder, values, format) {
    placeholder.forEach((v, i) => {
        if (new RegExp('(' + v + ')').test(format)) {
            let value = values[i].toString();
            if (v === timePlaceholder.year) {
                format = format.replace(RegExp.$1, value.toString().substr(4 - RegExp.$1.length));
            }
            else {
                format = format.replace(RegExp.$1, RegExp.$1.length === 1 ? value : ("00" + value).substr(value.length));
            }
        }
    });
    return format;
}
const secondsFormatPlaceholder = [timePlaceholder.day, timePlaceholder.hour, timePlaceholder.minute, timePlaceholder.seconds];
function secondsFormat(seconds, format = 'HH:mm:ss') {
    let d = Math.floor(seconds / secondsConstant.day);
    let h = Math.floor(seconds / secondsConstant.hour);
    let m = Math.floor(seconds / secondsConstant.minutes % secondsConstant.minutes);
    let s = Math.floor(seconds % secondsConstant.minutes);
    return replaceTimePlaceholder(secondsFormatPlaceholder, [d, h, m, s], format);
}
exports.secondsFormat = secondsFormat;
const dateFormatPlaceholder = [
    timePlaceholder.year, timePlaceholder.Q, timePlaceholder.month, timePlaceholder.day,
    timePlaceholder.hour, timePlaceholder.minute, timePlaceholder.seconds, timePlaceholder.milliseconds
];
function dateFormat(date, format) {
    const values = [
        date.getFullYear(),
        Math.floor((date.getMonth() + 3) / 3),
        date.getMonth() + 1,
        date.getDate(),
        date.getHours(),
        date.getMinutes(),
        date.getSeconds(),
        date.getMilliseconds() //毫秒 
    ];
    return replaceTimePlaceholder(dateFormatPlaceholder, values, format);
}
exports.dateFormat = dateFormat;
