import { KV } from '../../types';
import { IDisposable } from '../../common';
declare type FormatOptions<T> = {
    format?: (data: T) => KV;
    childrenField?: string;
    useTreeForamt?: boolean;
    sourceNode?: LinkedTreeNode<T>;
};
declare type ImportOptions<T> = {
    coverRoot?: boolean;
} & FormatOptions<T>;
export interface ILinkedTreeRaw<T> {
    data: T;
    children: ILinkedTreeRaw<T>[];
}
export declare class LinkedTreeNode<T> {
    _parent?: LinkedTreeNode<T>;
    _prev?: LinkedTreeNode<T>;
    _next?: LinkedTreeNode<T>;
    _firstChild?: LinkedTreeNode<T>;
    _lastChild?: LinkedTreeNode<T>;
    _children: LinkedTreeNode<T>[];
    data: T;
    get parent(): LinkedTreeNode<T> | undefined;
    get prev(): LinkedTreeNode<T> | undefined;
    get next(): LinkedTreeNode<T> | undefined;
    get firstChild(): LinkedTreeNode<T> | undefined;
    get lastChild(): LinkedTreeNode<T> | undefined;
    get children(): LinkedTreeNode<T>[];
    get isLeaf(): boolean;
    get hasParent(): boolean;
    constructor(data: T, clone?: boolean);
    hasAncestor(ancestor: LinkedTreeNode<T>): boolean;
    getNodeHeight(root?: LinkedTreeNode<T>): number;
}
export declare class LinkedTree<T> implements IDisposable {
    private _root;
    get root(): LinkedTreeNode<T>;
    constructor(data?: T);
    isRoot(target: LinkedTreeNode<T>): void;
    setRoot(data: T): LinkedTree<T>;
    setRoot(node: LinkedTreeNode<T> | T): LinkedTree<T>;
    addToBefore(source: LinkedTreeNode<T>, target: LinkedTreeNode<T>): this;
    addToAfter(source: LinkedTreeNode<T>, target: LinkedTreeNode<T>): this;
    addToChildrenFirst(source: LinkedTreeNode<T>, target: LinkedTreeNode<T>): this;
    addToChildrenLast(source: LinkedTreeNode<T>, target: LinkedTreeNode<T>): this;
    private _canMoveRemove;
    private _canMoveToSibling;
    moveToBefore(source: LinkedTreeNode<T>, target: LinkedTreeNode<T>): this;
    moveToAfter(source: LinkedTreeNode<T>, target: LinkedTreeNode<T>): this;
    moveToChildrenFirst(source: LinkedTreeNode<T>, target: LinkedTreeNode<T>): this;
    moveToChildrenLast(source: LinkedTreeNode<T>, target: LinkedTreeNode<T>): this;
    moveToReplace(source: LinkedTreeNode<T>, target: LinkedTreeNode<T>): this;
    separate(target: LinkedTreeNode<T>): this;
    remove(target: LinkedTreeNode<T>): this;
    isAncestor(source: LinkedTreeNode<T>, ancestor: LinkedTreeNode<T>): boolean;
    map(callback: (current: LinkedTreeNode<T>, children: any) => any): any;
    map(source: LinkedTreeNode<T>, callback: (current: LinkedTreeNode<T>, children: any) => any): any;
    bubble(source: LinkedTreeNode<T>, fn: (parent: LinkedTreeNode<T>) => boolean | void): this;
    import(source: KV | KV[], { sourceNode, format, childrenField, useTreeForamt, coverRoot }?: ImportOptions<T>): this;
    export({ sourceNode, format, childrenField, useTreeForamt }?: FormatOptions<T>): any;
    dispose(): this;
}
export {};
