"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.bubble = exports.loopDFSTail = exports.loopDFS = exports.destroy = exports.remove = exports.cutLinked = exports.setAfter = exports.setBefore = exports.setLast = exports.setFirst = exports.insert = void 0;
function insert(tree, target) {
    tree._parent = target;
    target._children.push(tree);
}
exports.insert = insert;
function setFirst(tree, target) {
    if (target._firstChild && target.children.length > 1) {
        tree._next = target._firstChild;
        target._firstChild._prev = tree;
        target._firstChild = tree;
    }
    else {
        target._firstChild = tree;
        target._lastChild = tree;
    }
    if (tree._prev)
        tree._prev = undefined;
}
exports.setFirst = setFirst;
function setLast(tree, target) {
    if (target._lastChild && target.children.length > 1) {
        target._lastChild._next = tree;
        tree._prev = target._lastChild;
        target._lastChild = tree;
    }
    else {
        target._firstChild = tree;
        target._lastChild = tree;
    }
    if (tree._next)
        tree._next = undefined;
}
exports.setLast = setLast;
function setBefore(source, target) {
    if (target._parent) {
        let prev = target._prev;
        source._next = target;
        if (prev) {
            source._prev = prev;
            prev._next = source;
            target._prev = source;
        }
        else {
            target._prev = source;
            target._parent._firstChild = source;
        }
    }
}
exports.setBefore = setBefore;
function setAfter(tree, target) {
    if (target._parent) {
        let next = target._next;
        tree._prev = target;
        if (next) {
            tree._next = next;
            next._prev = tree;
            target._next = tree;
        }
        else {
            target._next = tree;
            target._parent._lastChild = tree;
        }
    }
}
exports.setAfter = setAfter;
function cutPrev(tree) {
    //处理第一个节点
    if (tree._prev) {
        tree._prev._next = tree._next;
    }
    else {
        tree._parent._firstChild = tree._next;
        if (tree._next) {
            tree._next._prev = undefined;
        }
    }
}
function cutNext(tree) {
    if (tree._next) {
        tree._next._prev = tree._prev;
    }
    else {
        tree._parent._lastChild = tree._prev;
        if (tree._prev) {
            tree._prev._next = undefined;
        }
    }
}
function cutJoin(tree) {
    tree._prev = undefined;
    tree._next = undefined;
}
function cutLinked(tree, afterSibling) {
    let cuts = [tree];
    if (tree._parent) {
        let hasAfter = afterSibling && tree._parent._children.indexOf(afterSibling) > -1;
        cutPrev(tree);
        cutNext(hasAfter ? afterSibling : tree);
        cutJoin(hasAfter ? afterSibling : tree);
        let temp = tree;
        if (hasAfter) {
            while (temp !== afterSibling) {
                temp._parent = undefined;
                cuts.push(temp);
                temp = temp._next;
            }
            temp._parent = undefined;
            cuts.push(temp);
        }
        else
            tree._parent = undefined;
    }
    return cuts;
}
exports.cutLinked = cutLinked;
function remove(tree) {
    if (tree._parent) {
        let index = tree._parent._children.indexOf(tree);
        if (index > -1) {
            tree._parent._children.splice(index, 1);
            cutLinked(tree);
            return tree;
        }
    }
}
exports.remove = remove;
function destroy(tree) {
    remove(tree);
    tree.data = undefined;
    tree._firstChild = undefined;
    tree._lastChild = undefined;
    tree._children = [];
}
exports.destroy = destroy;
function loopDFS(tree, handle, handleResult) {
    let subTree = tree._firstChild;
    handleResult = (handle(tree, handleResult) || {});
    while (subTree && !handleResult.break) {
        loopDFS(subTree, handle, handleResult);
        subTree = subTree._next;
    }
}
exports.loopDFS = loopDFS;
function loopDFSTail(tree, handle) {
    let subTree = tree._firstChild, currentResults = [];
    while (subTree) {
        let prevResult = loopDFSTail(subTree, handle);
        if (prevResult) {
            currentResults.push(prevResult);
            if (prevResult.break)
                break;
        }
        subTree = subTree._next;
    }
    return handle(tree, currentResults);
}
exports.loopDFSTail = loopDFSTail;
function bubble(tree, handle) {
    let top = tree, complete = true;
    while (top._parent) {
        if (handle(top) === false) {
            complete = false;
            break;
        }
        top = top._parent;
    }
    return complete;
}
exports.bubble = bubble;
