"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildTreeByJoinNode = exports.loopBFS = exports.loopDFSTail = exports.loopDFS = void 0;
const tslib_1 = require("tslib");
const types_1 = require("../../common/types");
const object_1 = require("../../common/object");
const map_1 = require("../../common/map");
function loopDFS(tree, childFiled, handle, handleResult) {
    const children = (tree[childFiled] || []);
    handleResult = (handle(tree, handleResult) || {});
    if (children.length && !handleResult.break) {
        children.forEach(node => loopDFS(node, childFiled, handle, handleResult));
    }
}
exports.loopDFS = loopDFS;
function loopDFSTail(tree, childFiled, handle) {
    const children = (tree[childFiled] || []), currentResults = [];
    for (let i = 0; i < children.length; i++) {
        const prevResult = loopDFSTail(children[i], childFiled, handle);
        if (prevResult) {
            currentResults.push(prevResult);
            if (prevResult.break)
                break;
        }
    }
    return handle(tree, currentResults);
}
exports.loopDFSTail = loopDFSTail;
function loopBFS(tree, childFiledOrCallback, callback) {
    let loops = Array.isArray(tree) ? [...tree] : [tree];
    let childFiled = 'children', handle = childFiledOrCallback;
    if (types_1.isString(childFiledOrCallback)) {
        childFiled = childFiledOrCallback;
        handle = callback;
    }
    while (loops.length) {
        let current = loops.shift();
        if (handle(current) !== false)
            current[childFiled] && loops.push(...current[childFiled]);
    }
}
exports.loopBFS = loopBFS;
function buildTreeByJoinNode(source, _a) {
    var { childrenField = 'children', clearJoinNode = true } = _a, options = tslib_1.__rest(_a, ["childrenField", "clearJoinNode"]);
    const sourceMap = new Map(), treeData = [];
    map_1.mergeListToMap(sourceMap, source, options.key);
    sourceMap.forEach((n) => {
        if (n[options.joinNodeKey]) {
            const { right, parent } = n[options.joinNodeKey];
            const parentNode = parent && sourceMap.get(parent), rightNode = right && sourceMap.get(right);
            // 预设 子节点集合对象
            let children = treeData; // 没有父节点时，做为根处理
            if (parentNode) {
                // 线性数据是无序的，有可能先遍历到子节点再到父节点
                !Array.isArray(parentNode[childrenField]) && (parentNode[childrenField] = []);
                children = parentNode[childrenField];
            }
            // 上一段代码可能已经处理过空对象到情况，如果数组已经存在则不赋值
            !Array.isArray(n[childrenField]) && (n[childrenField] = []);
            // 处理 右边节点
            let index = rightNode ? children.findIndex(v => v[options.key] === rightNode[options.key]) : -1;
            index > -1 ? children.splice(index, 1, n, children[index]) : children.push(n);
            // 清除 连接节点
            clearJoinNode && options.joinNodeKey && delete n[options.joinNodeKey];
            // 包裹 指定属性
            options.wrapField && object_1.wrapObj(n, options.wrapField, [childrenField]);
        }
    });
    return treeData;
}
exports.buildTreeByJoinNode = buildTreeByJoinNode;
